/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with {library}; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package ch.hortis.sonar.web.listener;

import ch.hortis.sonar.core.batch.BatchFactory;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.sonar.commons.database.DatabaseConfiguration;
import org.sonar.commons.database.WrongDatabaseVersionException;
import org.sonar.commons.database.dialect.JdbcDialect;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NameClassPair;
import javax.naming.NameNotFoundException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.persistence.PersistenceException;
import javax.servlet.ServletContextEvent;
import javax.servlet.ServletContextListener;

/**
 * Servlet context listener to startup/stop sonar core
 */
public class SonarCoreStartupListener implements ServletContextListener {
  protected static final Logger LOG = LoggerFactory.getLogger(SonarCoreStartupListener.class);

  public void contextInitialized(ServletContextEvent ctxEvent) {
    Context ctx;
    try {
      ctx = new InitialContext();

    } catch (NamingException ex) {
      ctxEvent.getServletContext().log("Naming exception occured during JNDI initial context creation", ex);
      return;
    }

    DatabaseConfiguration dbConf = new DatabaseConfiguration(DatabaseConfiguration.DEFAULT_JNDI_NAME, null);
    String jdbcDialect = lookup(ctx, "java:comp/env/sonar/jdbc-dialect", null);
    if (jdbcDialect == null) {
      jdbcDialect = dbConf.autodetectJdbcDialect();
    } else {
      dbConf.setJdbcDialect(JdbcDialect.fromString(jdbcDialect));
    }
    SonarContext.getInstance().put(SonarContext.JDBC_DIALECT_CONTEXT_KEY, jdbcDialect);


    try {
      populateSonarContext(ctx);

    } catch (NameNotFoundException ex) {
      // no java:comp/env/sonar found.. not an issue
    } catch (NamingException ex) {
      ctxEvent.getServletContext().log("Naming exception occured during java:comp/env/sonar lookups", ex);
    }

    BatchFactory.init(dbConf);

    try {
      BatchFactory.start();

    } catch (WrongDatabaseVersionException e) {
      LOG.warn(e.getMessage());

    } catch (PersistenceException e) {
      LOG.warn("Database must be created or updated. Please browse to your sonar server.", e);
    }

    try {
      ctx.close();
    } catch (NamingException e) {
      // should log a warning
    }

  }

  private void populateSonarContext(Context ctx) throws NamingException {
    SonarContext sonarContext = SonarContext.getInstance();
    NamingEnumeration<NameClassPair> rorEnv = ctx.list("java:comp/env/sonar");
    while (rorEnv.hasMoreElements()) {
      NameClassPair pair = rorEnv.nextElement();
      Object val = ctx.lookup("java:comp/env/sonar/" + pair.getName());
      if (val == null) val = "";
      sonarContext.put(pair.getName(), val.toString());
    }
  }

  public void contextDestroyed(ServletContextEvent ctxEvent) {
    BatchFactory.stop();
    SonarContext.getInstance().destroy();
  }

  private <T> T lookup(Context ctx, String key, Object defaultValue) {
    Object value;
    try {
      value = ctx.lookup(key);
    } catch (NamingException e) {
      value = defaultValue;
    }
    return (T) value;
  }
}